<?php

namespace jrosset\Collections;

/**
 * Interface for a collection
 *
 * @psalm-template     TKey of array-key
 * @template-covariant TValue
 * @template-extends IImmutableCollection<TKey, TValue>
 */
interface ICollection extends IImmutableCollection {
    /**
     * Set a value to a key
     *
     * @param array-key    $key   The key
     * @param mixed        $value The value
     *
     * @psalm-param TKey   $key
     * @psalm-param TValue $value
     *
     * @return $this
     */
    public function set ($key, $value): self;
    /**
     * Add a value
     *
     * @param mixed        $value The value
     *
     * @psalm-param TValue $value
     *
     * @return $this
     */
    public function add ($value): self;

    /**
     * Empties the collection
     *
     * @return $this
     */
    public function clear (): self;
    /**
     * Delete a key
     *
     * @param array-key  $key The key
     *
     * @psalm-param TKey $key
     *
     * @return $this
     */
    public function remove ($key): self;
    /**
     * Delete all instances of a value
     *
     * @param mixed        $value  The value
     * @param bool         $strict Strict comparison ?
     *
     * @psalm-param TValue $value
     *
     * @return $this
     */
    public function removeValue ($value, bool $strict = false): self;
}